# Background information

This repository consists of all code developt during the bachelor thesis "Exploring possibilities for Gitlab as a Learning Management System" in the bachelor program Digital Infrastructure and Cyber Security (2019 - 2022).  
The aim of this thesis was to explore to which degree alternative systems available at NTNU could solve the main challenges of a learning Management System (LMS).
A learning management system (LMS) is a software application for the administration, documentation, tracking, reporting, automation, and delivery of educational courses, training programs, or learning and development programs.

## Team information

Nicholas Bodvis Sellevåg <br>
| nicholbs@stud.ntnu.no

Oddbjørn Sivert Borge-Jensen <br>
| oddbjosb@stud.ntnu.no

Fabian Kongelf <br>
| fabiako@stud.ntnu.no 

Yan Senko <br>
| yans@stud.ntnu.no

## What is contained within the repository?
The repository contains several projects that are effectively independent projects serving different functionalities of the developed system in relation to the thesis. For example, this repository transforms markdown documents into websites with Gitlab Pages automations. The folders "cypress_gitlab_pages", "auto-git-structure" and "webpage-accessebility-test" are all respectively their own repository and can be interconnected with CI/CD pipelines ad downstream and upstream projects. As a result, proper installation of the coherent system developed begins with configuring the "Gitlab pages" repository. Afterwards each other project should be created seperately. Excluding the mentioned seperate projects all other files and folders are related to the configurable Gitlab Pages system.

Group members recommend creating an overarching group, then add each project within that group.
See the other projects repository/folder for their installation guide.
A list of which folder was a seperate project is further outlined in "other projects" section.

The original repository and working example can be viewed at [Department of Computer Science in Trondheim Gitlab instance](https://gitlab.stud.idi.ntnu.no/bachelor1/Website) within the next few months as of writing 19/05/2022. Furthermore, the resulting webpages would be accessible at [Gitlab Pages](https://bachelor1.pages.stud.idi.ntnu.no/Website/) 


### Installation of the GitLab Pages
The gitlab pages project require a small amount of configuration before working as intended. 

1. Configure _config.yml file for the Static Site Generator Jekyll Service
Variables including baseurl, title and description affect the generated pages. Other variables such as "announcement_webhook" affect integrations with Teams. 

It is notewhorty that most of the configuration is done beforehand and does not need changing. Therefore the required changes for users are marked with REQUIRED in the comment section.

2. Configure .gitlab-ci.yml for Gitlab CI/CD pipelines
There are five stages created by group members and can be interchanged or removed completely depending on the users wishes.
The possible stages are as follows in .gitlab-ci.yml file:
- test                     #Code-quality testing on Gitlab pages repository
- vuln-scan                #Snyk dependency scanning of repository
- deploy                   #Gitlab Pages
- web-testing              #Pre defined cypress tests against Gitlab Pages
- web-accessibility-report #accessibility testing of Gitlab Pages
- notification             #Send web request to defined Teams webhook for publishing changes in repository

For example, users are free to remove the stages from the YAML file. On the other hand, to work as intended the Gitlab Pages project should triggers two downstream projects for running Web Unit tests with Cypress JavaScripted framework and WCAG testing as Gitlab's own feature. One could make all stages part of this repository through combining the different .gitlab-ci.yml and overwriting the stages from YAML file in each project folder into this repository's version.

To solely have Gitlab Pages transform markdown documents into websites simply remove all other stages than deploy in this repository's .gitlab-ci.yml file. 

### Other projects
The following folder are thier own projects:
- auto-git-structure
- cypress_gitlab_pages
- webpage-accessebility-test

### Local development
To be able to develop or experiment with the project locally without having to commit every edit and wait for the ci/cd to execute the code create a local development environment.

First, install [Ruby](https://rubyinstaller.org/downloads/) locally  

On the left side in the table “With DEVKIT” choose “Ruby+Devkit 2.7.5-1 (x64)”. Click through the installation guild.

When Ruby is finished installing you can confirm it's installation in a terminal with the command “ruby -v”, confirme you possess the ruby version 2.7. 

In a teminal at the root of the project, do the following commands: 
- gem install bundler jekyll	//instals jekyll and move dependencies in “Gemfile” 
- bundle install		     //instals all dependencies from“Gemfile” 
- bundle exec jekyll serve	//Run Jekyll and host _site localt, the output is the local URL of the website 

## Dependencies
Jekyll Static Site Generator
[Jekyll Ruby plugin](https://rubygems.org/gems/jekyll)

Create automated table of contents from headings in markdown documents
[Jekyll-toc Ruby plugin](https://rubygems.org/gems/jekyll-toc)

Access environment variables using Liquid tags with Jekyll
[Jekyll-environment-variables Ruby plugin](https://rubygems.org/gems/jekyll-environment-variables)

A Jekyll plugin to generate an Atom feed of your Jekyll posts. In practice RSS generation.
[Jekyll-feed Ruby plugin](https://rubygems.org/gems/jekyll-feed)

Time Date of posts
[Tzinfo-data Ruby plugin](https://rubygems.org/gems/tzinfo-data)

This plugin takes any file that is in posts folders, and copy them to the folder in which the post HTML page will be created.
[Jekyll-postfiles Ruby plugin](https://rubygems.org/gems/jekyll-postfiles)

# How the website works
The Static Site Generator [Jekyll](https://jekyllrb.com/) transforms markdown documents provided in "content" folder into corresponding websites. There are several scripted automations to make the process simple as possible for users. In short, Jekyll transforms markdown to HTML/CSS. Gitlab CI/CD is configured to run scripts that moves and creates the necessary files and folder structures. As a result, Gitlab is provided with website documents hosted as Gitlab Pages. 

The following chapters elaborates further on the developed solution. But for a full comprehension of the system users should read the bachelor thesis "Exploring possibilities for Gitlab as a Learning Management System" published in [NTNU Open](https://ntnuopen.ntnu.no/).

## Front matter

Added to the top of a markdown post to set certain specifications. Most of the typical content of the front matter is either set automatically or have default values, but in case you should want to change anything manually, here are the defaults and syntax:

>\-\-\-  
>layout: post  
>toc: true  
>\-\-\-  

## Layout

Defines the layout to use for a post. For now all posts should use the standard layout "post", as seen in *_layouts/post.html*.
In practice, layouts are templates with some basic html that includes the content of a given page, in addition a standarized footer and header all available in the "/webpage/_includes" folder. 

## Table of Contents

Posts will automatically generate a table of contents based on headings present in the source markdown file. Tocs are enabled by default, but can be disabled for a single post by adding the following line to the front matter of the post:

### Example
>\-\-\-  
>toc: false  
>\-\-\-  

## Syntax highlighting

To render a code block with syntax highlighting, surround your code as follows:

>{% highlight c %}  
>  
>#include <stdio.h>  
>  
>int main() {  
>&ensp;&ensp;printf("Hello, World!");  
>&ensp;&ensp;return 0;  
>}  
>  
>{% endhighlight %}   

The argument to the highlight tag (C in the example above) is the language identifier. You can view a list of all supported languages and their identifiers [here](https://github.com/rouge-ruby/rouge/blob/master/docs/Languages.md).

## Directory/folder structure

All documents to become websites and relating images should be created inside the "content" folder in its own subfolder. This subfolder will automatically be made into a category by Jekyll.

For example:

```
content/
    myFolder/
        -> markdown.md
        -> image.png
        -> document.pdf
    anotherFolder/
        -> markdownToBeWebsite2.md
        -> imageToWebsite2.png
        -> imageToWebsite2.pdf
```

This is not in and of itself a valid Jekyll structure, but deploy.sh will take the contents of the "content" folder and copy it into a temp folder "webpage/content", rearranging into a proper structure in the process. Should you as a user want to define the entire structure manually, simply adding the required _posts folders to the original "content" folder will stop the script from making any changes. 

```
content/
    myFolder/
        _posts/
            -> markdown.md
            -> image.png
            -> document.pdf
    anotherFolder/
        _posts/
            -> markdownToBeWebsite2.md
            -> imageToWebsite2.png
            -> imageToWebsite2.pdf
```

As long as an image or pdf is in the same folder as a markdown file, you can refer to them directly:

```
![My image](image.png)
```

If you want one or more assets to be accessed by several markdown files in several different folders, we recommend adding it to the "assets" folder. Referencing files from this folder requires using the absolute path starting at the baseurl:

```
![My image](/nameOfRepo/webpage/assets/image.png)
```

### Plugins

DO NOT REMOVE THE _plugins FOLDER! It might be empty, but plugins won't work without it!


