#!/bin/bash

# reading _config.yml and finding webhook attribute
while read line; do

    # check if line from _config has attribute name
    if [[ $line == "announcement_webhook"* ]]; then

        # assigning webhook attribute in _config to webhook variable
        webhook=${line:21}
    fi
done < _config.yml

announce_dir=($(grep '^announcedir: ' _config.yml | awk '{ print $2 }' | cut -d '"' -f 2))

# Checks if webhook variables length is greater then zero (did the prev while loop find a webhook)
if [[ -n $webhook ]]; then
    # declering a nummerator and array
    declare -i n=0
    facts=''

    # reading from txt file generated by git ci/cd containing meta data for commit.
    while read line; do

        # nummerator shows whitch line is read
        if [ $n -eq 0 ]; then
            message=$line
        fi

        if [ $n -eq 1 ]; then
            time_stamp=$line
        fi

        if [ $n -eq 2 ]; then
            web_url=$line
        fi

        if [ $n -gt 3 ];  then
            if [[ $line == *"|"* ]]; then
                IFS='|' read -r -a array <<< $line
                facts+='{"name": "'"${array[0]}"'", "value": "'"${array[1]}"'"},'
            
                # if file is part has announcements directory as part of its path
                if [[ ${array[0]} == *"/$announce_dir/"* ]]; then
                    # add all of a files content to variable
                    announce_content=$( cat ${array[0]} )
                
                    # add date line to announcement
                    sed -i '1i---\n custom-date: '$time_stamp'\n---' ${array[0]}
                fi
            fi 
        fi

        # increas nummerator
        n+=1
    done < announce_meta.txt    

    # message sendt to teams.
    if [[ ! -z "$announce_content" ]]; then
        output=$(curl --location --request POST $webhook \
        --header 'Content-Type: application/json' \
        --data-raw '{
            "@type": "MessageCard",
            "@context": "http://schema.org/extensions",
            "themeColor": "0076D7",
            "summary": "'"$message"'",
            "sections": [{
                "activityTitle": "'"$message"'",
                "activitySubtitle": "'"$time_stamp"'",
                "activityImage": "https://about.gitlab.com/images/press/logo/png/gitlab-icon-rgb.png",
                "type": "TextBlock",
                "text": "'"$announce_content"'",
                "markdown": true
            },{ 
                "facts": ['"$facts"'],
                "markdown": true
            },{
                "facts": [{
                    "name": "URL",
                    "value": "'"$web_url"'"
                }]
            }]
        }')
    else
        output=$(curl --location --request POST $webhook \
        --header 'Content-Type: application/json' \
        --data-raw '{
            "@type": "MessageCard",
            "@context": "http://schema.org/extensions",
            "themeColor": "0076D7",
            "summary": "'"$message"'",
            "sections": [{
                "activityTitle": "'"$message"'",
                "activitySubtitle": "'"$time_stamp"'",
                "activityImage": "https://about.gitlab.com/images/press/logo/png/gitlab-icon-rgb.png",
                "facts": ['"$facts"'],
                "markdown": true
            },{
                "facts": [{
                    "name": "URL",
                    "value": "'"$web_url"'"
                }]
            }]
        }')
    fi

    if [[ $output != '1' ]]; then
        echo -e "\033[0;31m !-- Webhook error --! \033[0m"
        echo $output
    fi
else
    echo 'Missing announcements webhook-link, see https://docs.microsoft.com/en-us/microsoftteams/platform/webhooks-and-connectors/how-to/add-incoming-webhook to create webhook, add your webhook-link in _config.yml to auto send announcements to ms teams.'
fi
